#! /usr/bin/perl
use strict;
use File::Path;
my $VERSION = '0.1';
my $DIR = "/usr/local/bin/colit";
my ($yyyy, $mm, $dd);
my ($fileNumber, $stationNumber);
my $OUT = "/usr/local/bin/colit/out";
my $IN = $DIR."/source";
my ($aregexp, $bregexp);
my @AR;
my @BR;

my @d = (	"wBillVersion","wSSID","dwBillID","bRecType","bPartRecID","wSeqNum","bOpProp","sdnOpNo","OpNet","OpLata", 			"","bOutpOpProp","sdnOutpOpNo","OutpOpNet","OutpOpLata","","bDialTpProp","sdnDialTpNo","DialTpNet","DialTpLata","",
		"bDialTpPreLen","bTpProp","sdnTpNo","TpNet","TpLata","","bTpNoPreLen","bOutpTpProp","sdnOutpTpNo","OutpTpNet",
		"OutpTpLata","","bOutpTpPreLen","tAnswerTime","","bServiceCat","tEndTime","","bEndReason","bOpType","bFlag","bInTrkGrpType",
		"tInTrkGrpNo","tInTrkCrctNo","tInTrkConnectTime","","tInTrkDisConnectTime","","OrigClli","","","OrigMem","","","bInMGType","wInMGID",
		"bOpSSIPAddr","","","","bOpMGIPAddr","","","","bOpRtpIPAddr","","","","bOpProtocol","bCallDirect","bCallType","bCoding","bCallParty",
		"bOutTrkGrpType","tOutTrkGrpNo","tOutTrkCircuitNo","tOutTrkConnectTime","","tOutTrkDisConnectTime","","TermClli","","",
		"TermMem","","","wOutMGType","wOutMGID","bTpSSIPAddr","","","","bTpMGIPAddr","","","","bTpRtpIPAddr","","","","bTpProtocol","dwFaxPage","bSS",
		"bChargeID","bLinkProp","sdnLinkNo","LinkNet","LinkLata","","dwFee","dwCustomerID","dwCustLocationID","bAccountCodeType",
		"bAccountCode","tAccessNumber","tCarrierID","wOpCtxNo","wTpCtxNo","tIngressBytes","tEgressBytes","bAuthorityType",
		"Reserved5","Reserved6","tAuthorityCode","wCarrierSelInfo","bBearerSvc","bTeleSvc","bUSS1","bUSS3","tSpOpNo",
		"tSpTpNo","bBillProp","sdnBillNo","BillNet","BillLata","","bTransNoProp","sdnTransNo","TransNet","TransLata","","bLocaNoProp",
		"sdnLocaNo","LocaNet","LocaLata","","wChgRateKind","bChgModulatorType","bChgModulatorVal","bInAttachFeeKind","dwInAttachFeeVal","bTransParam");


my $help = "usage: convert.pl [key] [param]
 -i input directory, default $IN
 -o output directory, default $OUT
 -a regexp for A-phone, default none
 -b regexp for B-phone, default none";
my $var="1";
foreach my $arg (@ARGV)
{
  if( $var eq "-i")
  {
	$IN = $arg;
	$var = "1";
  }
  elsif($var eq "-o")
  {
	$OUT = $arg;
	$var = "1";
  }
  elsif($var eq "-a")
  {
	@AR = split(/;/, $arg);
	$var = "1";
  }
  elsif($var eq "-b")
  {
	@BR = split(/;/, $arg);
	$var = "1";
  }
  elsif($arg =~ /-i|-o|-a|-b/)
  {
	$var = $arg;
  }
}

if($ARGV[0] eq "--help")
{
  print "$help\n";
  exit(0);
}

print "@AR\n";
print "@BR\n";

opendir (DIR,"$IN");
foreach my $cur_file (readdir(DIR))
{
  # обрабатываем все файлы
  if( $cur_file =~ /^CDR_(\d{4})(\d{2})(\d{2})_(\d{4}).ZTE_(\d{6})$/ )  
  {
	($yyyy, $mm, $dd, $fileNumber, $stationNumber) = ($1, $2, $3, $4, $5);
	
	mkpath("$OUT/$yyyy/$mm/" );

	my $out_file = $cur_file;
	$out_file .= '.txt';

	open FILE_OUT, ">$OUT/$yyyy/$mm/$out_file" || die "Can't open $out_file on write!\n";

	parse("$cur_file");
	close FILE_OUT;	
  }
  elsif( not $cur_file =~ /(.gz$|^\.+$)/ )
  {
	my $out_file = $cur_file.".txt";
	#$out_file =~ s/(dis|di)/txt/;

	open FILE_OUT, ">$OUT/$out_file" || die "Can't open $out_file on write!\n";

	parse("$cur_file");
	close FILE_OUT;	
  }
}
closedir (DIR);

#
# Парсим сырой файл.
#
sub parse
{
  my $file = shift;
  open(FILE,"<:bytes:raw", "$IN/$file") || die "Ошибка при открытии $file. $!\n";
  
  my $block;
  my @c;
  my $i;
  my $format_str2 = #'H4H4H8ccH4ch64ch2h2ch64H2h2h2ch64H2h2h2H2ch64H2h2h2H2ch64H2h2h2H2H10H2H10H2H2bbbbbbbbh2H4H4H10H10H4H2H4H2H2H4H8H8H8H2H2H2H2H2H2H4H4H10H10H4H2H4H2H2H4H8H8H8H2H8H14H2ch64H2h2h2nH8H8H2H20H8H4H4H4H16H16H2H10H0H32H4ccH2H2H10H10ch64H2h2h2ch64H2h2h2ch64H2h2h2H4H2H4H2H8H40';
'H4nH8CCnCh64Ch2h2Ch64Ch2h2Ch64Ch2h2B8Ch64Ch2h2H2Ch64Ch2h2H2NCH2NCH2C'.
'B8H2nnNCNCCCCCCCCnCCCCCCCCCCCCCCCCCCnnNCNCCCCCCCH2nCCCCCCCCCCCCCNH14CCh64'.
'Ch2h2nNNCH20H8H4nnQQH2H10H4H32nCCCCH10H10Ch64Ch2h2Ch64Ch2h2Ch64Ch2h2nCnCNH40';
  my $time2000 = 946684800;

#	read(FILE, $block, 560);
  while( read(FILE, $block, 559))
  {
	@c = unpack($format_str2, $block);
	#print "@c\n";
	#print "@c\n";
	my $res='';
	my $str="";
	for($i = 0; $i < scalar(@c); $i++)
	{
		if($d[$i] =~ /OpLata|OutpOpLata|TpLata|OutpTpLata|LinkLata|BillLata|TransLata|LocaLata/)
		{	
			$res = $c[$i+1].$c[$i];
			$res =~ s/0+$//;
		}
		elsif($d[$i] =~ /sdnOpNo|sdnOutpOpNo|sdnDialTpNo|sdnTpNo|sdnOutpTpNo|sdnLinkNo|tSpOpNo|tSpTpNo|sdnBillNo|sdnTransNo|sdnLocaNo/ )
		{ 
			$res= sprintf('%s', $c[$i]);
			$res =~ tr/0//d;
	        	$res =~ tr/a/0/;
		}
		elsif($d[$i] =~ /tAnswerTime|tEndTime|tInTrkConnectTime|tInTrkDisConnectTime|tOutTrkConnectTime|tOutTrkDisConnectTime/ )
		{ 
			$res = $c[$i] + $time2000;
			my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst,$milisec) = gmtime($res);
			$year += 1900;
			$mon = ++$mon<10 ? '0'.$mon:$mon;
			$mday = $mday<10 ? '0'.$mday:$mday;
			$hour = $hour<10 ? '0'.$hour:$hour;
			$min = $min<10 ? '0'.$min:$min;
			$sec = $sec<10 ? '0'.$sec:$sec;
			$milisec = $c[$i+1]<10 ? '0'.$c[$i+1]:$c[$i+1];
			$res = $year.'-'.$mon.'-'.$mday.' '.$hour.':'.$min.':'.$sec.'.'.$milisec.'0';
		}
		elsif($d[$i] =~ /bOpSSIPAddr|bOpMGIPAddr|bOpRtpIPAddr|bTpSSIPAddr|bTpMGIPAddr|bTpRtpIPAddr/)
		{
			$res = $c[$i].'.'.$c[$i+1].'.'.$c[$i+2].'.'.$c[$i+3];
		}
		elsif($d[$i] =~ /OrigMem|OrigClli|TermClli|TermMem/)
		{
			$res = $c[$i].'-'.$c[$i+1].'-'.$c[$i+2];
		}
		elsif($d[$i] =~ /wBillVersion|dwBillID|tCarrierID|bSS|bTransParam/)
		{
			$res = '0x'.$c[$i];
		}
		elsif($d[$i] =~ /bAccountCode|tAccessNumber|tAuthorityCode|Reserved5/)
		{
			$res = '0' if ($c[$i] == 0);
		}
		else
		{
			$res = $c[$i];
		}
		
		if($i == 0)
		{		
			print FILE_OUT "$res";
		}

		if($d[$i] ne "" && $i > 0)
		{
			#print "$i $d[$i]:\t\t\t$res\n";
			print FILE_OUT ",$res";
		}

	}
	print FILE_OUT "\n";
	#print "0:$c[0] 1:$c[1] 2:$c[2] 3:$c[3] 4:$c[4] 5:$c[5] 6:$c[6] 7:$c[7] 8:$c[8] 10-9:$c[10]$c[9] 11:$c[11] 12:$c[12] 13:$c[13]\n";
	#print FILE_OUT "$c[1] $c[2] $c[3] $yyyy-$c[6]-$c[5] $c[9]:$c[8]:$c[7] $c[12]:$c[11]:$c[10] $c[13] $c[15] $c[16] $c[17] $c[18]\n";
  }
  close(FILE);
	
}



