#! /usr/bin/perl
use strict;

my $DIR = "/usr/local/bin/ngn";
my $CFGFILE = "ngn.cfg";
my $logfile = "ngn.log";
my $cronfile = "crontab.file";
my $response;

# put, zip
my ($logpath, $rotate, $compression, $compressionlevel, $schedule) = ($DIR."/log", "daily", "gzip", 9, "* * * *" );

parse_cfg("$DIR/$CFGFILE");
addcrontask();

#
# Parsing the configuration file
#
sub parse_cfg
{
  my $file = shift;

  open(CFGFILE, "$file") || die "ERROR Ошибка при открытии $file. $!";
  print "INFO  File '$file' open\n";

  my @contents = <CFGFILE>;
  close(CFGFILE);

  foreach( @contents )
  {
	  $logpath = $1 if(/^logpath=(.*);$/);
	  $rotate = $1 if(/^rotate=(daily|weekly|monthly)$;/);
	  $compression = $1 if(/^compression=(gzip|tar);$/);
	  $compressionlevel = $1 if(/^compressionlevel=(\d+);$/);
	  $schedule = $1 if(/^schedule="(.*)";$/);
  }
	
  if( $logpath eq ""  )
  {
	print "ERROR Некоректный конфигурационный файл! Считанные параметры: logpath=$logpath;\n";
	exit 0;
  }
  else
  {
	print "INFO  Успешно считаны параметры:  logpath=$logpath; rotate=$rotate; compression=$compression; compressionlevel=$compressionlevel;\n";
	print "INFO  Файл '$file' успешно обработан\n";
  }
}


sub addcrontask
{
        print "Добавляем записи в cron. Создаем файл $DIR/$cronfile\n";
        open(CRONFILE, ">$DIR/$cronfile") || die "Ошибка при открытии $DIR/$cronfile. $!\n";
  		my @oldcrontab = `crontab -l`;
        if( $#oldcrontab > -1 )
        {
          foreach(@oldcrontab)
          {
                print CRONFILE "$_" if(!/ngn/ && !/archive.pl/);
          }
        }
        else
        {
          print CRONFILE "SHELL=/bin/sh\n\# colit: Task download\n";
  		}

        # добавляем коллектор
        print "Добавляем новую задачу: 0 $schedule flock -xn $DIR/ngn.lock -c '$DIR/ngn.pl' >> $logpath/ngn.log 2>&1\n";
        print CRONFILE "0 $schedule flock -xn $DIR/ngn.lock -c '$DIR/ngn.pl' >> $logpath/ngn.log 2>&1\n";


        # архивация логов, всех файлов с названием $logfile
        my $task;
	    $task = "30 0 * * *" if($rotate eq "daily"); # каждый день в 00:30
        $task = "30 0 * * 0" if($rotate eq "weekly"); # каждое воскресение в 00:30
        $task = "30 0 1 * *" if($rotate eq "monthly"); # каждый месяц 1 числа в 00:30
        print "Добавляем новую задачу: $task $DIR/archive.pl $compression $compressionlevel $logpath $logfile n  >> /dev/null 2>&1\n";
        print CRONFILE "# ngn: Compressing logs\n";
        print CRONFILE "$task $DIR/archive.pl $compression $compressionlevel $logpath $logfile n  >> /dev/null 2>&1\n";
        close(CRONFILE);
        print "Создан файл $DIR/$cronfile\n";
        $response = `crontab $DIR/$cronfile`;
        print "Задания добавлены в cron: $response\n";

}

sub delcrontask
{
        print "Удаляем записи из cron.";
	    my @oldcrontab = `crontab -l`;
        if( $#oldcrontab > -1 )
        {
      	  print "Создаем файл $DIR/$cronfile.";
          open(CRONFILE, ">$DIR/$cronfile") || die "Ошибка при открытии $DIR/$cronfile. $!\n";
          foreach(@oldcrontab)
          {
                print CRONFILE "$_" if(!/ngn/ && !/archive.pl/);
          }
          close(CRONFILE);
          print "Создан файл $DIR/$cronfile";
          $response = `crontab $DIR/$cronfile`;
          print "Задания удалены из cron";
          $response = `rm $DIR/$cronfile`;
        }
}
